﻿// FreeFileMemory.jsx

/*
Закрытие и открытие документа — чтобы обнулить память, которую забрал индизайн на хранение откатов.
С какого-то момента этот объем становится тормозом в работе программы.
Идея этого скрипта в выполнении шести шагов, после прохождения которых индизайн должен продолжить работу с текущим файлом быстрее.
Точнее сказать — продолжить работу с такой же скоростью, какая была бы при первом открытии этого файла.
Вот эти шаги:
1. Сохранить текущий файл и запомнить открытую страницу.
2. Сохранить файл в формате idml.
3. Закрыть текущий indd файл.
4. Открыть только что созданный idml файл. У него еще нет имени, и он открыт на первой странице.
5. Переименовать или удалить сохранённый indd файл. Вариант 'переименовать' или 'удалить' определяется состоянием переменной 'remove', она ниже по тексту.
    Если переменная в состоянии false, то этот файл после сохранения будет переименован: его расширение будет '.indd#'.
    Если она true, то после открытия idml файла сохранённый indd файл будет удалён.
6. Дать текущему файлу имя исходного indd файла и перейти на ту страницу, где был запущен этот скрипт.

Итак, если индизайн стал заметно тормозить, просто запустите этот скрипт. 
После завершения работы изменится формат отображения: страница или разворот будут на весь экран, и индизайн будет дальше работать с этим файлом так же быстро, как при первом его открытии.
*/

// Версия 10.11.2017
//
// © Михаил Иванюшин, 2017  m.ivanyushin@gmail.com

var programTitul = "Закрытие и открытие файла как способ повысить быстродействие индизайна";

var remove = true; // если false, то текущий файл после сохранения будет переименован: его расширение будет '.indd#'
                              // если true, то после открытия idml-файла этот файл будет удалён
try {
    var myDoc = app.activeDocument;
    }
catch (e) {
    alert ("Нет открытых документов.", programTitul);
    exit();
    }
try { var myDocPath = myDoc.filePath; } // если документ не сохранён, маршрут не сообщается
catch(e) {
	alert("Это ещё ни разу не сохранённый файл.\nДайте ему имя.", programTitul);	
	exit();     
}
app.scriptPreferences.enableRedraw = true; //  эта строка актуальна только для макинтоша:   https://forums.adobe.com/message/4619984#4619984    https://forums.adobe.com/message/6921564#6921564
var fullName = decodeURI(myDoc.fullName);
var docName = decodeURI(myDoc.name);
var win = new Window ("palette", programTitul, undefined,{closeButton: false});
win.alignChildren = "left";
var lineSize = [0,0,650,35];
var infoLine1 = win.add("statictext",lineSize, "");
if (!remove) var infoLine2 = win.add("statictext",lineSize, "");
infoLine1.text = "(1/6) Сохранение текущего файла";
win.show();
win.update(); // Исключительно для владельцев маков. 
                    // По какой-то неясной пока причине в облачных версиях индизайна на некоторых макинтошах созданные cкриптами окна перестали обновляться.
                    // Возможно, это связано с вариантом операционной системы на маке. 
                    // На abobe community  я увидел оператор update() как  решение этой редкой проблемы в оформлении классического прогресс-бара: https://forums.adobe.com/message/4619984#4619984
                    // Коль скоро в этом скрипте содержимое переменной типа "statictext" играет роль прогресс-бара, я просто решил подстраховаться.
                    // Для машин под Windows этот костыль не нужен.              
myDoc.save(new File(fullName));
var idmlFile = new File(fullName.split(".indd")[0]+ ".idml");
var renName = fullName.split(".indd")[0]+ ".indd.#";
infoLine1.text = "(2/6) Сохранение текущего файла в формате idml";
win.update();
myDoc.exportFile(ExportFormat.INDESIGN_MARKUP, idmlFile);
var myPage = app.windows[0].activePage.documentOffset; // текущая рабочая страница
infoLine1.text = "(3/6) Закрытие текущего файла";
win.update();
myDoc.close(SaveOptions.NO); 
var text46 = "";
if (remove) text46 = "(4/6) Переименование текущего файла" else text46 = "(4/6) Удаление текущего файла";
infoLine1.text = text46;
win.update();
if (!remove) File (fullName).rename(renName) else try { File (fullName).remove() } catch (e) { };
infoLine1.text = "(5/6) Открытие сохранённого idml файла";
if (!remove) {
    var text56 = "";
    text56 = "Файл, из которого получен idml файл, теперь имеет расширение '.indd.#'. Его можно удалить.";
    infoLine2.text = text56;
    }
win.update();
var idmlDoc = app.open(new File(idmlFile), true); 
infoLine1.text = "(6/6) Сохранение этого файла с прежним именем [ " + docName + " ]" ;
win.update();
idmlDoc.save(new File(fullName));
// Непонятно, почему после этого сохранения перед именем файла появляется звёздочка, будто этот файл изменён и пока не сохранён.
// Только что открытый idml файл такой звёздочки не имеет, а тут операция сохранения, и она поднимает флажок, что в файле есть изменения!
// Да, изменилось имя. Но с этим новым именем и сохранили.
// Пробую охранить ещё раз: вновь открытые файлы не должны иметь этой звёздочки...
//app.activeDocument.save(new File(fullName));  // ... нет, фокус не удался, повторное сохранение не помогло, звёздочка осталась. Поэтому это повторное сохранение отключено.
//
app.activeWindow.activePage = app.activeDocument.pages[myPage]; // перейдём на страницу, которая была активной в момент запуска этого скрипта
app.documents[0].layoutWindows[0].zoom(ZoomOptions.FIT_SPREAD);
infoLine1.text = "Файл обновлён, ресурсы памяти возвращены в начальное состояние. Можно продолжать работу" ; //  :-))  Наши цели ясны, задачи определены. За работу, товарищи!  — Н. С. Хрущев
if (!remove) infoLine2.text = "";
win.update();
$.sleep(3000);
win.close();
exit();
